%splot.m (enhanced subplot). Creates a highly customized subplot grid using
%a minimum number of parameters (the remaining parameters are computed by
%the program).
% Warning: error handling is minimalistic and the program is certainly not
% bug free.
% Note: when using units other than 'normalized', the result on screen
% might be very bad but the eps (or any other format) output should be OK.
% 
% Syntax
% 
% [ax_handle,fig_handle] = splot(n,m);
% [ax_handle,fig_handle] = splot(n,m,...,PropertyName,PropertyValue,...);
% [ax_handle,fig_handle] = splot(n,m,width,height,...,PropertyName,PropertyValue,...);
% 
% Description 
%
% [ax_handle,fig_handle] = splot(n,m); similar to subplot(n,m,k) : creates
% a subplot grid with n rows and m columns. ax_handle is a matrix of
% handles corresponding to each subplot axes. ax_handle(i,j) is the handle
% of the subplot box at the ith row and jth column. "axes(ax(i,j))" selects
% the corresponding subplot box as the current axes object. fig_handle is
% the handle of the figure containing the subplots.
%
% [ax_handle,fig_handle] = splot(n,m,...,PropertyName,PropertyValue,...)
% allows the specification of additional properties. See the Properties section below.
%
% [ax_handle,fig_handle] =
% splot(n,m,width,height,...,PropertyName,PropertyValue,...) same as
% splot(n,m,...,PropertyName,PropertyValue,...) but also defines the
% dimensions of the figure (using units defined by the 'units' property).
% Note: specifying the dimensions is mostly useful when the figure is to be
% printed to an eps file (or other format). It is therefore highly
% recommended to specify units other than 'normalized' in this case. Width
% and/or height parameters can be specified as empty matrices in which case
% the program will attempt to compute them using the other parameters.
% Rem: the eps figure output produced by matlab (using the print command)
% is generally slightly smaller thant the dimensions specified because
% Matlab automatically removes empty spaces around the edges of the figure.
%
%
% Properties
%
% 'units'             : units used for all specified lengths
%                       default: 'normalized'. (see available matlab units for possible values)
% 'AR'                : aspect ratio of the figure (AR=width/height)
%                       default: undefined
% 'figure'            : handle of the figure where the subplots shall be created
%                       default: creates a new figure
% 'scale'             : scale factor. All specified lengths are multiplied by the scale factor. This is mostly useful to reduce or increase the spacing of dashes (or dots etc) in lines produced by the plot command with the '--' line style. 
% Warning: the default thickness of the lines and the default font sizes are not multiplied by the scale factor. These must be specified independently.
%                       default: 1
% 'spacing'           : spacing between the subplots. Can be either scalar or a two element vector corresponding to horizontal and vertical spacing respectively.
%                       default: undefined (see spacing_p)
% 'xspacing'          : horizontal spacing between subplots. Can be either scalar or a vector of length m-1 specifying a variable horizontal spacing between each column.
%                       default: undefined (see xspacing_p)
% 'yspacing'          : vertical spacing between subplots. Can be either scalar or a vector of length n-1 specifying a variable vertical spacing between each row.
%                       default: undefined (see yspacing_p)
% 'edges'             : size of the outer margins surrounding the subplot grid. The value can be either a string containing one or more of the letters 'l', 'r', 't', 'b' (for left, right, top and bottom) or a vector of length 1, 2 or 4.
%                       If the value is a string, predefined sizes margins are added to the edges specified by the string. The predefined sizes are 8% of the width or height for the left, right and top edges and 1% for the right edge (because there are generally no labels on the right hand side of the figure).
%                       If the value is a numeric vector, it specifies either all outer margins (length=1), the horizontal and vertical outer margins (length=2) or the 4 outer margins with the ordering: left, right top and bottom (length=4). 
%                       default: [0,0,0,0]
% 'axeswidth'         : widths of the subplots. Can be either a scalar (all columns have the same width) or a vector of length m specifying the width of each column.
%                       default: undefined
% 'axesheight'        : heights of the subplots. Can be either a scalar (all rows have the same height) or a vector of length n specifying the height of each row.
%                       default: undefined
% 'spacing_p'         : same as 'spacing' but proportional to the dimensions of the figure.
%                       default: 0.05
% 'xspacing_p'        : same as 'xspacing' but proportional to the width of the figure.
%                       default: 0.05
% 'yspacing_p'        : same as 'yspacing' but proportional to the height of the figure.
%                       default: 0.05
% 'edges_p'           : same as 'edges' but proportional to the dimensions of the figure.
%                       default: [0,0,0,0]
% 'axeswidth_p'       : same as 'axeswidth' but proportional to the width of the figure.
%                       default: undefined
% 'axesheight_p'      : same as 'axesheight' but proportional to the height of the figure.
%                       default: undefined
% 'axesAR'            : aspect ratios of the subplots (scalar).
%                       default: undefined
% 'FigureProperties'  : cell array containing PropertyName/PropertyValue pairs forwarded to the figure obect. Useful for instance to define default font sizes for the figure.
%                       default: {}
% 'AxesProperties'    : cell array containing PropertyName/PropertyValue pairs forwarded to the axes obects. Useful for instance to define default line widths.
%                       default: {}
%
% Examples
%
% Create simple subplots (for display on screen)
% ax_handles = splot(3,4); % creates the subplots (3 rows 4 columns)
% axes(ax_handles(2,3))    % select the subplot at the 2nd row and 3rd columns
% plot(x)                  % plot something in this subplot
%
% Create an eps figure 6 inches wide and 4 inches high containing 12 subplots (3 rows x 4 columns), with 0.5 in outer margins (for labels) and 0.1 in spacing between subplots:
% [ax_handles,fig_handle] = splot(3,4,6,4,'units','inches','edges',[.5,.01,.5,.5],'spacing',.1);
% ...                               % some commands to plot something in the subplots
% savefig(fig_handle,'name.eps')    % save the eps figure
%
%
% G. Rilling - Last modification 2009

%%% French documentation: 
%
%splot.m (subplot ameliore). Permet de personnaliser a volonte une grille
% de subplots en precisant un minimum de parametres (les autres etant
% calcules automatiquement).
% Attention : la gestion des erreurs est minimaliste et le programme
% comporte certainement encore des bugs.
% Note : le resultat a l'ecran peut etre atroce lorsque des unites
% differentes de 'normalized' sont precisees, mais la sortie en eps (ou
% autre) est generalement OK.
% 
% Syntaxe
% 
% [ax_handle,fig_handle] = splot(n,m);
% [ax_handle,fig_handle] = splot(n,m,...,PropertyName,PropertyValue,...);
% [ax_handle,fig_handle] = splot(n,m,largeur,hauteur,...,PropertyName,PropertyValue,...);
% 
% Description 
%
% [ax_handle,fig_handle] = splot(n,m); similaire a subplot(n,m,k) : cree
% une grille de subplots n lignes x m colonnes. ax_handle est la matrice
% des handles vers chacun des subplots : ax_handle(i,j) est le pointeur
% vers le subplot de la i-eme ligne et j-ieme colonne (
% "axes(ax_handle(i,j))" permet de selectionner ce subplot comme subplot
% courant). fig_handle est le handle vers la figure.
%
% [ax_handle,fig_handle] = splot(n,m,...,PropertyName,PropertyValue,...)
% permet de specifier des proprietes supplementaires. Voir liste des proprietes.
%
% [ax_handle,fig_handle] =
% splot(n,m,largeur,hauteur,...,PropertyName,PropertyValue,...) idem
% splot(n,m,...,PropertyName,PropertyValue,...) mais specifie en plus les
% dimensions de la figure (avec les unites definies par la propriete
% 'units' (cf Proprietes optionnelles)). Il est d'ailleurs fortement
% conseille de specifier 'units' different de 'normalized' si largeur et
% hauteur sont differents de 1. largeur et hauteur peuvent egalement etre
% vides ([]) auquel cas le programme tentera de les calculer en fonction
% des autres parametres fournis.
% Rem : la figure sauvegardee par matlab (via print) est generalement un peu 
% plus petite parce que matlab coupe automatiquement les espaces vides sur 
% les bords.
%
%
% Proprietes
%
% 'units'             : unites utilisees pour toutes les longueurs specifiees. 
%                       defaut : 'normalized'. (voir les unites dispos dans 
%                       matlab pour les differentes valeurs possibles)
% 'AR'                : rapport d'aspect de la figure (AR=largeur/hauteur)
%                       defaut : indefini
% 'figure'            : handle de la figure ou les subplots doivent etre crees.
%                       defaut : cree une nouvelle figure
% 'scale'             : facteur d'echelle. Toutes les longueurs specifiees sont
%                       multipliees par le facteur d'echelle. Permet de jouer par
%                       exemple sur l'espacement des tirets ou tirets-points des
%                       lignes obtenues par "plot". Attention, les epaisseurs par
%                       defaut des lignes et les tailles de fontes par defaut ne
%                       sont pas multipliees par scale. defaut : 1
% 'marges'            : marges entre les subplots. La valeur peut etre soit scalaire
%                       soit un vecteur a 2 elements correspondant (dans l'ordre)
%                       aux marges horizontales et aux marges verticales.
%                       defaut indefini (cf marges_p)
% 'margex'            : marges horizontales entre les subplots. La valeur peut etre
%                       soit scalaire (marges identiques entre chaque subplot) soit
%                       un vecteur de longueur m-1 specifiant des marges variables
%                       entre les colonnes. defaut indefini (cf marges_p)
% 'margey'            : marges verticales entre les subplots. La valeur peut etre soit
%                       scalaire (marges identiques entre chaque subplot) soit un
%                       vecteur de longueur n-1 specifiant des marges variables entre
%                       les rangees. defaut indefini (cf marges_p)
% 'bords'             : dimensions des marges a l'exterieur de la grille de subplots.
%                       La valeur peut etre soit une chaine de caracteres composees
%                       des lettres 'h' 'b' 'g' 'd' (pour haut,bas,gauche,droite),
%                       soit un vecteur de 1, 2 ou 4 longueurs.
%                       Dans le premier cas, des marges externes de taille predefinie
%                       sont ajoutees sur les bords presents dans la chaine de
%                       caracteres. Les tailles predefinies sont 8% de la largeur (ou
%                       hauteur) de la figure pour les bords haut, bas et gauche et 1%
%                       pour le bord droit (ou il n'y a generalement pas de label).
%                       Dans le cas ou la valeur est un vecteur de nombres, elle
%                       specifie, soit toutes les marges externes (longueur 1), soit
%                       les marges "gauche et droite" et "haut et bas" (longueur 2),
%                       soit les 4 marges dans l'ordre "gauche, droite, haut, bas"
%                       (longueur 4). defaut [0,0,0,0]
% 'largeuraxes'       : largeurs des subplots. La valeur peut etre soit scalaire pour
%                       des subplots de largeurs identiques soit un vecteur de
%                       longueur m pour des largeurs variables. defaut indefini
% 'hauteuraxes'       : hauteurs des subplots. La valeur peut etre soit scalaire pour
%                       des subplots de hauteurs identiques soit un vecteur de longueur n
%                       pour des hauteurs variables. defaut indefini
% 'marges_p'          : idem marges mais proportionnellement aux dimensions de la figure.
%                       defaut 0.05
% 'margex_p'          : idem margex mais proportionnellement a la largeur de la figure.
%                       defaut 0.05
% 'margey_p'          : idem margey mais proportionnellement a la hauteur de la figure.
%                       defaut 0.05
% 'bords_p'           : idem bords mais proportionnellement aux dimensions de la figure.
%                       defaut [0,0,0,0]
% 'largeuraxes_p'     : idem largeuraxes mais proportionnellement a la largeur de la figure.
%                       defaut indefini
% 'hauteuraxes_p'     : idem hauteuraxes mais proportionnellement a la hauteur de la figure.
%                       defaut indefini
% 'ARaxes'            : rapports d'aspect des subplots. Valeur scalaire. defaut indefini
% 'FigureProperties'  : cell array contenant des paires proprietes/valeur transmises a la
%                       figure. Utile par exemple pour definir des tailles de fontes par defaut.
%                       defaut {}
% 'AxesProperties'    : cell array contenant des paires proprietes/valeur transmises aux
%                       subplots. Utile par exemple pour definir des epaisseurs de lignes par defaut.
%                       defaut {}
%
% Exemples
%
% creation de subplots simples (pour affichage a l'ecran)
% ax_handles = splot(3,4); % creation des subplots (3 lignes 4 colonnes)
% axes(ax_handles(2,3))  % selection du subplot en 2eme ligne 3eme colonne
% plot(x)                % commande de plot
%
% creer une figure eps de 6 pouces par 4 avec une grille de 3 lignes x 4 colonnes, des marges 
% externes d'un demi pouce (pour des eventuels xlabel, ylabel title...) et des marges internes
% d'un dixieme de pouce :
% [ax_handles,fig_handle] = splot(3,4,6,4,'units','inches','bords',[.5,.01,.5,.5],'marges',.1);
% ...
% savefig(fig_handle,'nom.eps')


function [axes_handles,figure_handle] = splot(varargin)

Properties = {'Marges','Margex','Margey','Bords','Units','Scale','AxesProperties','Figure','AxesAR','FigureAR'};
% DefaultMargeProportion = 0.1;
DefaultUnits = 'normalized';
DefaultScale = 1;
% DefaultRightMarginProportion = 0.01;
% DefaultLeftMarginProportion = 0.12;
% DefaultTopMarginProportion = 0.12;
% DefaultBottomMarginProportion = 0.12;
Default_margeg_p = 0;
Default_marged_p = 0;
Default_margeh_p = 0;
Default_margeb_p = 0;
Default_on_margeg_p = 0.08;
Default_on_marged_p = 0.01;
Default_on_margeh_p = 0.08;
Default_on_margeb_p = 0.08;
Default_margex_p = 0.05;
Default_margey_p = 0.05;
Default_largeur_axes_p = 0.9;
Default_hauteur_axes_p = 0.9;

char_inds = find(cellfun(@ischar,varargin));

N = 1;
M = 1;
largeur = 1;
hauteur = 1;

if isempty(char_inds)
  n_numeric_args = length(varargin);
else
  n_numeric_args = char_inds(1)-1;
end
switch n_numeric_args
  case 0 %defaut
  case 1
    M = varargin{1};
  case 2
    [N,M] = deal(varargin{1:2});
  case 3
    [N,M,largeur] = deal(varargin{1:3});
    hauteur = [];
  case 4
    [N,M,largeur,hauteur] = deal(varargin{1:4});
  otherwise
    error('Too many numeric arguments')
end
if isempty(char_inds)
  varargin = {};
else
  varargin = varargin(char_inds(1):end);
end

if mod(length(varargin),2)
  error('bad syntax')
end
  
axes_handles = zeros(N,M);

if isempty(largeur), largeur = NaN;end
if isempty(hauteur), hauteur = NaN;end

warning off splot:experimental


[margex,margey,margeg,marged,margeh,margeb,largeur_axes,hauteur_axes,units,scale,onscreen_pos,axes_properties,figure_properties,FigHandle] = parseinput();

largeur = largeur*scale;
hauteur = hauteur*scale;
margex = margex*scale;
margey = fliplr(margey*scale);
margeg = margeg*scale;
marged = marged*scale;
margeb = margeb*scale;
margeh = margeh*scale;
largeur_axes = largeur_axes*scale;
hauteur_axes = fliplr(hauteur_axes*scale);


% largeur_axes = (largeur-margeg-marged-(M-1)*margex)/M;
% hauteur_axes = (hauteur-margeb-margeh-(N-1)*margey)/N;

if FigHandle
  fig_h = FigHandle;
  figure(FigHandle)
else
  fig_h = figure;
end

% position on screen
if any(~isnan(onscreen_pos))
  set(fig_h,'units','pixels');
  P=get(fig_h,'Position');
  P(~isnan(onscreen_pos))=onscreen_pos(~isnan(onscreen_pos));
  monitor_resolution=get(0,'MonitorPositions');
  P(3:4)=min(P(3:4),monitor_resolution(3:4));
  P(1:2)=min(P(1:2),monitor_resolution(3:4)-P(3:4));
  set(fig_h,'Position',P);
end

% position on paper
if ~strcmpi(units,'normalized')
  set(fig_h,'Units',units,'PaperUnits',units,'PaperSize',[largeur+4,hauteur+4],'PaperPosition',[2,2,largeur,hauteur],figure_properties{:});
end


for i = 0:N-1
  for j = 1:M
    indsj = logical([ones(1,j-1),zeros(1,M-j)]);
    indsi = logical([ones(1,i),zeros(1,N-i-1)]);
    axes_handles(N-i,j) = axes('Units',units,'Position',[margeg+sum(margex(indsj))+sum(largeur_axes(logical([indsj,0]))),margeb+sum(margey(indsi))+sum(hauteur_axes(logical([indsi,0]))),largeur_axes(j),hauteur_axes(N-i)],axes_properties{:});
  end
end

if nargout > 1
  figure_handle = fig_h;
end

  function [margex,margey,margeg,marged,margeh,margeb,largeur_axes,hauteur_axes,units,scale,onscreen_pos,axes_properties,figure_properties,FigHandle] = parseinput()
    %         if M > 1
    %             margex = DefaultMargeProportion*largeur/(M-1);
    %         else
    %             margex = 0;
    %         end
    %         if N > 1
    %             margey = DefaultMargeProportion*hauteur/(N-1);
    %         else
    %             margey = 0;
    %         end
    margex = NaN*zeros(1,M-1);
    margey = NaN*zeros(1,N-1);
    margeb = NaN;
    margeh = NaN;
    margeg = NaN;
    marged = NaN;
    largeur_axes = NaN*zeros(1,M);
    hauteur_axes = NaN*zeros(1,N);
    AR_axes = NaN*zeros(N,M);
    AR_figure = NaN;
    margex_p = NaN*zeros(1,M-1);
    margey_p = NaN*zeros(1,N-1);
    margeb_p = NaN;
    margeh_p = NaN;
    margeg_p = NaN;
    marged_p = NaN;
    largeur_axes_p = NaN*zeros(1,M);
    hauteur_axes_p = NaN*zeros(1,N);
    largeur_axes_p_X = NaN;
    hauteur_axes_p_X = NaN;
    margex_p_X = NaN;
    margey_p_X = NaN;
    units = DefaultUnits;
    scale = DefaultScale;
    axes_properties = {};
    figure_properties = {};
    onscreen_pos=NaN*zeros(1,4);
    args = varargin;
    FigHandle = 0;
    if mod(length(args),2)
      error('bad argument syntax')
    end
    while ~isempty(args)
      if ~isa(args{1},'char')
        error('bad argument syntax')
      end
      val = args{2};
      switch lower(args{1})
        case {'onscreen_size','taille_ecran'}
          if ~isvector(val) && length(val)==2
            error('onscreen_size must be a length 2 vector')
          end
          onscreen_pos(3:4)=val(:)';
        case {'onscreen_position','position_ecran'}
          if ~isvector(val) && length(val)==4
            error('onscreen_position must be a length 4 vector')
          end
          onscreen_pos=val(:)';
        % longueurs d\ufffdfinies de mani\ufffdre absolue
        case {'marges','spacing'}
          if ~isscalar(val) && length(val) ~= 2
            error('bad argument syntax')
          end
          if length(val) == 1 && ~isnan(val)
            margex = repmat(val,1,M-1);
            margey = repmat(val,1,N-1);
          else
            if ~isnan(val(1))
              margex = repmat(val(1),1,M-1);
            end
            if ~isnan(val(2))
              margey = repmat(val(2),1,N-1);
            end
          end
        case {'margex','xspacing'}
          if isscalar(val) && length(val) ~= M-1
            margex = repmat(val,1,M-1);
          elseif length(val) == M-1
            ind = ~isnan(val);
            margex(ind) = val(ind);
          else
            error('bad argument syntax')
          end
        case {'margey','yspacing'}
          if isscalar(val) && length(val) ~= N-1
            margey = repmat(val,1,N-1);
          elseif length(val) == N-1
            ind = ~isnan(val);
            margey(ind) = val(ind);
            margey = fliplr(margey);
          else
            error('bad argument syntax')
          end
        case {'bords','edges'}
          if isa(val,'char')
            val=lower(val);
            if ~isempty(strfind(val,'g')) || ~isempty(strfind(val,'l'))
              margeg_p = Default_on_margeg_p;
            end
            if ~isempty(strfind(val,'d')) || ~isempty(strfind(val,'r'))
              marged_p = Default_on_marged_p;
            end
            if ~isempty(strfind(val,'h')) || ~isempty(strfind(val,'t'))
              margeh_p = Default_on_margeh_p;
            end
            if ~isempty(strfind(val,'b'))
              margeb_p = Default_on_margeb_p;
            end
          else
            if length(val) == 1 && ~isnan(val)
              margeg = val;
              marged = val;
              margeh = val;
              margeb = val;
            elseif length(val) == 2
              if ~isnan(val(1))
                margeg = val(1);
                marged = val(1);
              end
              if ~isnan(val(2))
                margeh = val(2);
                margeb = val(2);
              end
            elseif length(val) == 4
              if ~isnan(val(1))
                margeg = val(1);
              end
              if ~isnan(val(2))
                marged = val(2);
              end
              if ~isnan(val(3))
                margeh = val(3);
              end
              if ~isnan(val(4))
                margeb = val(4);
              end
            else
              error('bad argument syntax')
            end
          end
        case {'largeuraxes','axeswidth'}
          if isscalar(val) && ~isnan(val)
            largeur_axes = repmat(val,1,M);
          elseif length(val) == M
            ind = ~isnan(val);
            largeur_axes(ind) = val(ind);
          else
            error('bad argument syntax')
          end
        case {'hauteuraxes','axesheight'}
          if isscalar(val) && ~isnan(val)
            hauteur_axes = repmat(val,1,N);
          elseif length(val) == N
            ind = ~isnan(val);
            hauteur_axes(ind) = val(ind);
          else
            error('bad argument syntax')
          end
          % longueurs d\ufffdfinies par proportions de la figure totale
        case {'marges_p','spacing_p'}
          if ~isscalar(val) && length(val) ~= 2
            error('bad argument syntax')
          end
          if length(val) == 1 && ~isnan(val)
            margex_p = repmat(val,1,M-1);
            margey_p = repmat(val,1,N-1);
          else
            if ~isnan(val(1))
              margex_p = repmat(val(1),1,M-1);
            end
            if ~isnan(val(2))
              margey_p = repmat(val(2),1,N-1);
            end
          end
        case {'margex_p','xspacing_p'}
          if isscalar(val) && length(val) ~= M-1
            margex_p = repmat(val,1,M-1);
          elseif length(val) == M-1
            ind = ~isnan(val);
            margex_p(ind) = val(ind);
          else
            error('bad argument syntax')
          end
        case {'margey_p','yspacing_p'}
          if isscalar(val) && length(val) ~= N-1
            margey_p = repmat(val,1,N-1);
          elseif length(val) == N-1
            ind = ~isnan(val);
            margey_p(ind) = val(ind);
          else
            error('bad argument syntax')
          end
        case {'bords_p','edges_p'}
          if isa(val,'char')
            val=lower(val);
            if ~isempty(strfind(val,'g')) || ~isempty(strfind(val,'l'))
              margeg_p = Default_on_margeg_p;
            end
            if ~isempty(strfind(val,'d')) || ~isempty(strfind(val,'r'))
              marged_p = Default_on_marged_p;
            end
            if ~isempty(strfind(val,'h')) || ~isempty(strfind(val,'t'))
              margeh_p = Default_on_margeh_p;
            end
            if ~isempty(strfind(val,'b'))
              margeb_p = Default_on_margeb_p;
            end
          else
            if length(val) == 1 && ~isnan(val)
              margeg_p = val;
              marged_p = val;
              margeh_p = val;
              margeb_p = val;
            elseif length(val) == 2
              if ~isnan(val(1))
                margeg_p = val(1);
                marged_p = val(1);
              end
              if ~isnan(val(2))
                margeh_p = val(2);
                margeb_p = val(2);
              end
            elseif length(val) == 4
              if ~isnan(val(1))
                margeg_p = val(1);
              end
              if ~isnan(val(2))
                marged_p = val(2);
              end
              if ~isnan(val(3))
                margeh_p = val(3);
              end
              if ~isnan(val(4))
                margeb_p = val(4);
              end
            else
              error('bad argument syntax')
            end
          end
        case {'largeuraxes_p','axeswidth_p'}
          if isscalar(val) && ~isnan(val)
            largeur_axes_p = repmat(val,1,M);
          elseif length(val) == M
            ind = ~isnan(val);
            largeur_axes_p(ind) = val(ind);
          else
            error('bad argument syntax')
          end
        case {'hauteuraxes_p','axesheight_p'}
          if isscalar(val) && ~isnan(val)
            hauteur_axes_p = repmat(val,1,N);
          elseif length(val) == N
            ind = ~isnan(val);
            hauteur_axes_p(ind) = val(ind);
          else
            error('bad argument syntax')
          end
          % autres options
        case {'araxes','axesar'}
          if isscalar(val) && ~isnan(val)
            AR_axes = repmat(val,N,M);
          elseif all(size(val) == [M,N])
            AR_axes = val; %v\ufffdrification \ufffd impl\ufffdmenter
          elseif any(size(val) == 1) && length(val) == M
            ind = ~isnan(val);
            AR_axes(:,ind) = repmat(val(ind),N,1);
          elseif any(size(val) == 1) && length(val) == N
            ind = ~isnan(val);
            AR_axes(:,ind) = repmat(val(ind)',1,M);
          else
            error('bad argument syntax')
          end
        case {'largeur','width'}
          if ~isnan(val)
            largeur = val;
          end
        case {'hauteur','height'}
          if ~isnan(val)
            hauteur = val;
          end
        case 'ar'
          AR_figure = val;
        case 'units'
          units = val;
        case 'scale'
          scale = val;
        case 'figure'
          FigHandle = val;
        case 'axesproperties'
          axes_properties = val;
        case 'figureproperties'
          figure_properties = val;
        otherwise
          error(['unknown argument ',args{1}])
      end
      if length(args) > 2
        args = args(3:end);
      else
        args = [];
      end
    end
    % traitement des entr\ufffdes

    %% cas longueur1 = longueur2*proportion

    %construction de la liste
    liste = {'largeur','hauteur','AR_figure'};
    for i = 1:N
      for j = 1:M
        liste(end+1,:) = {sprintf('largeur_axes(%u)',j),sprintf('hauteur_axes(%u)',i),sprintf('AR_axes(%u,%u)',i,j)};
      end
    end
    % les 5 lignes suivantes sont incoh\ufffdrentes avec les pr\ufffdc\ufffddentes. ou alors rajouter AR_axes_p
    %     for i = 1:N
    %       for j = 1:M
    %         liste(end+1,:) = {sprintf('largeur_axes_p(%u)',j),sprintf('hauteur_axes_p(%u)',i),sprintf('AR_axes(%u,%u)',i,j)};
    %       end
    %     end
    liste(end+1:end+2,:) = {'margeg','largeur','margeg_p';'marged','largeur','marged_p'};
    for j = 1:M-1
      liste(end+1,:) = {['margex(',int2str(j),')'],'largeur',['margex_p(',int2str(j),')']};
      liste(end+1,:) = {sprintf('largeur_axes(%u)',j),'largeur',sprintf('largeur_axes_p(%u)',j)};
    end
    liste(end+1,:) = {['largeur_axes(',int2str(M),')'],'largeur',['largeur_axes_p(',int2str(M),')']};
    liste(end+1:end+2,:) = {'margeh','hauteur','margeh_p';'margeb','hauteur','margeb_p'};
    for i = 1:N-1
      liste(end+1,:) = {['margey(',int2str(i),')'],'hauteur',['margey_p(',int2str(i),')']};
      liste(end+1,:) = {sprintf('hauteur_axes(%u)',i),'hauteur',sprintf('hauteur_axes_p(%u)',i)};
    end
    liste(end+1,:) = {['hauteur_axes(',int2str(N),')'],'hauteur',['hauteur_axes_p(',int2str(N),')']};


    %traitement
    inconnues_largeur = {};
    equation_largeur = '-largeur+';
    inconnues_hauteur = {};
    equation_hauteur = '-hauteur+';
    for k = 1:size(liste,1)
      l1 = char(liste(k,1));
      l2 = char(liste(k,2));
      p = char(liste(k,3));
      s = find(isnan([eval(l1),eval(l2),eval(p)]));
      if isempty(s)
        warning('splot:redundancy','redundancy in the arguments')
      elseif length(s) == 1
        if s==1
          eval([l1,' = ',l2,'*',p,';']);
        elseif s==2
          eval([l2,' = ',l1,'/',p,';']);
        else
          eval([p,' = ',l1,'/',l2,';']);
        end
        if k<=2+2*M+N*M && k > 1+N*M % longueurs en x
          equation_largeur = [equation_largeur,p,'*largeur+'];
        elseif k>2+2*M+N*M && k <=3+2*M+2*N+N*M  % longueurs en y
          equation_hauteur = [equation_hauteur,p,'*hauteur+'];
        end
      else
        warning('splot:experimental','The program might fail to compute the dimensions of the figure.')
        if k == 1
          if find(s==1)
            inconnues_largeur(end+1) = {l1};
          end
          if find(s==2)
            inconnues_hauteur(end+1) = {l2};
          end
        elseif k<=2+2*M+N*M && k > 1+N*M % longueurs en x
          if find(s==2) % largeur inconnue -> on rajoute la quantit\ufffd connue dans l'\ufffdquation
            if find(s==1) % -> longueur inconnue
              if find(s==3) % longueur -> equation+inconnue
                equation_largeur = [equation_largeur,p,'*largeur+'];
                inconnues_largeur(end+1) = {p};
              else %-> on rajoute la proportion
                equation_largeur = [equation_largeur,p,'*largeur+'];
              end
            else % -> on rajoute la longueur
              equation_largeur = [equation_largeur,l1,'+'];
            end
          else % longueur et proportion inconnues -> on rajoute la longueur dans l'\ufffdquation et dans les inconnues
            equation_largeur = [equation_largeur,p,'*largeur+'];
            inconnues_largeur(end+1) = {p};
          end
        elseif k>2+2*M+N*M && k <=3+2*M+2*N+N*M  % longueurs en y
          if find(s==2) % hauteur inconnue -> on rajoute la quantit\ufffd connue dans l'\ufffdquation
            if find(s==1) % -> longueur inconnue
              if find(s==3) % longueur -> equation+inconnue
                equation_hauteur = [equation_hauteur,p,'*hauteur+'];
                inconnues_hauteur(end+1) = {p};
              else %-> on rajoute la proportion
                equation_hauteur = [equation_hauteur,p,'*hauteur+'];
              end
            else % -> on rajoute la longueur
              equation_hauteur = [equation_hauteur,l1,'+'];
            end
          else % longueur et proportion inconnues -> on rajoute la longueur dans l'\ufffdquation et dans les inconnues
            equation_hauteur = [equation_hauteur,p,'*hauteur+'];
            inconnues_hauteur(end+1) = {p};
          end
        end
      end
    end
    
    %v\ufffdrification de redondances \ufffdventuelles
    %         if ~isnan(largeur)&&any(~isnan(margex)&~isnan(margex_p))...
    %                 any(~isnan(largeur_axes)&~isnan(largeur_axes))...
    %                 |all(~isnan([margeg,margeg_p])...
    %                 |all(~isnan([marged,marged_p])
    %
    %
    %         |any(~isnan(margey)&~isnan(margey_p))...
    %             |all(~isnan([margeh,margeh_p])...
    %             |all(~isnan([margeb,margeb_p])...
    %
    %         margex = NaN*zeros(1,M-1);
    %         margey = NaN*zeros(1,N-1);
    %         margeb = NaN;
    %         margeh = NaN;
    %         margeg = NaN;
    %         marged = NaN;
    %         largeur_axes = NaN*zeros(1,M);
    %         hauteur_axes = NaN*zeros(1,N);
    %         AR_axes = NaN*zeros(N,M);
    %         AR_figure = NaN;
    %         margex_p = NaN*zeros(1,M-1);
    %         margey_p = NaN*zeros(1,N-1);
    %         margeb_p = NaN;
    %         margeh_p = NaN;
    %         margeg_p = NaN;
    %         marged_p = NaN;
    %         largeur_axes_p = NaN*zeros(1,M);
    %         hauteur_axes_p = NaN*zeros(1,N);
    %         units = DefaultUnits;
    %         scale = DefaultScale;
    %         axes_properties = {};
    %         figure_properties = {};

    % remplissage des trous par les valeurs par d\ufffdfaut
    
    % mettre \ufffd jour les hauteur_axes_p si AR_axes d\ufffdfini -> done -> puis supprim\ufffd parce que c'\ufffdtait con \ufffd la base

    %ajouter test pour voir dans quelle direction faire les calculs en premier
    
    
    if strcmpi('hauteur',inconnues_hauteur{1})
    
      fill_input_largeur;
    
      calcul_equation_largeur;
    
      %passer en longueurs absolues pour les largeurs
      prop_2_abs_largeur;
    
      %mettre \ufffd jour les hauteurs en cas de AR_axes non nuls
      maj_ARs;
    
      fill_input_hauteur;
      
      calcul_equation_hauteur;
    
      prop_2_abs_hauteur;
    else
      fill_input_hauteur;
    
      calcul_equation_hauteur;
    
      %passer en longueurs absolues pour les largeurs
      prop_2_abs_hauteur;
    
      %mettre \ufffd jour les hauteurs en cas de AR_axes non nuls
      maj_ARs;
    
      fill_input_largeur;
      
      calcul_equation_largeur;
    
      prop_2_abs_largeur;
    end

    
    function maj_ARs
      liste = {'largeur','hauteur','AR_figure'};
      for i = 1:N
        for j = 1:M
          liste(end+1,:) = {sprintf('largeur_axes(%u)',j),sprintf('hauteur_axes(%u)',i),sprintf('AR_axes(%u,%u)',i,j)};
        end
      end
      for k = 1:size(liste,1)
        l1 = char(liste(k,1));
        l2 = char(liste(k,2));
        p = char(liste(k,3));
        s = find(isnan([eval(l1),eval(l2),eval(p)]));
        if isempty(s)
          % warning('splot:redondance','redondance dans les arguments')
        elseif length(s) == 1
          if s==1
            eval([l1,' = ',l2,'*',p,';']);
          elseif s==2
            eval([l2,' = ',l1,'/',p,';']);
          else
            eval([p,' = ',l1,'/',l2,';']);
          end
          if k==1
            ind=strmatch('largeur',inconnues_largeur,'exact');
            if ~isempty(ind)
              tmp = true(1,length(inconnues_largeur));tmp(ind)=false;
              inconnues_largeur = inconnues_largeur(tmp);
            end
            ind=strmatch('hauteur',inconnues_hauteur,'exact');
            if ~isempty(ind)
              tmp = true(1,length(inconnues_hauteur));tmp(ind)=false;
              inconnues_hauteur = inconnues_hauteur(tmp);
            end
          else
            ind=strmatch(['largeur_axes_p',l1(13:end)],inconnues_largeur,'exact');
            if ~isempty(ind)
              tmp = true(1,length(inconnues_largeur));tmp(ind)=false;
              inconnues_largeur = inconnues_largeur(tmp);
              equation_largeur = regexprep(equation_largeur,['largeur_axes_p\(',l1(14:end-1),'\)\*largeur'],['largeur_axes',l1(13:end)]); % rem: un regexptranslate('escape',.) aurait aussi march\ufffd (mais plus lent)
            end
            ind=strmatch(['hauteur_axes_p',l2(13:end)],inconnues_hauteur,'exact');
            if ~isempty(ind)
              tmp = true(1,length(inconnues_hauteur));tmp(ind)=false;
              inconnues_hauteur = inconnues_hauteur(tmp);
              equation_hauteur = regexprep(equation_hauteur,['hauteur_axes_p\(',l2(14:end-1),'\)\*hauteur'],['hauteur_axes',l2(13:end)]); % rem: un regexptranslate('escape',.) aurait aussi march\ufffd (mais plus lent)
            end
          end
          %           if k<=2+2*M+2*N*M && k > 1+2*N*M % longueurs en x
          %             equation_largeur = [equation_largeur,p,'*largeur+'];
          %           elseif k>2+2*M+2*N*M && k <=3+2*M+2*N+2*N*M  % longueurs en y
          %             equation_hauteur = [equation_hauteur,p,'*hauteur+'];
          %           end
        else
          % 
        end
      end
      % mettre \ufffd jour les \ufffdquations et inconnues : soit au moment de la mise \ufffd jour soit tout d'un coup apr\ufffds (ce qui revient \ufffd recalculer les \ufffdquations ??)
    end

    
    % en largeur :
    function fill_input_largeur
      if length(inconnues_largeur)>1 % -> remplissage par les valeurs par d\ufffdfaut
        Ninconnues = length(inconnues_largeur);
        ind = strmatch('margeg_p',inconnues_largeur,'exact');
        if Ninconnues >1 && ~isempty(ind)
          margeg_p = Default_margeg_p;
          inconnues_largeur(ind) = [];
          Ninconnues = Ninconnues-1;
        end
        ind = strmatch('marged_p',inconnues_largeur,'exact');
        if Ninconnues >1 && ~isempty(ind)
          marged_p = Default_marged_p;
          inconnues_largeur(ind) = [];
          Ninconnues = Ninconnues-1;
        end
        ind = strmatch('margex_p(',inconnues_largeur)';
        if ~isempty(ind)
          tmp = regexp(cell2mat(inconnues_largeur(ind)),'\((\d+)\)','tokens');
          inds = str2num(char(cellfun(@char,tmp','UniformOutput',false)))';
          %                 first = eval(tmp(10:end-1));
          %                 inds = first+ind-ind(1);
          if length(ind) == Ninconnues % -> 1 seule inconnue pour tous les ind\ufffdfinis
            % inconnues_largeur = arraysub(inconnues_largeur,[],ind,ind);
            inconnues_largeur = {'margex_p_X'};
            margex_p_X = NaN;
            Ninconnues = 1;
            str = regexprep(int2str(inds),'\W*','|'); % on transforme les indices en string "1|2|4"
            equation_largeur = regexprep(equation_largeur,['margex_p\((',str,')\)'],'margex_p_X');
          else % il y a d'autres inconnues -> on bouche les trous sans se poser de question
            margex_p(inds) = Default_margex_p/(M-1);
            Ninconnues = Ninconnues -length(inds);
            inconnues_largeur(ind) = [];
          end
        end

        ind = strmatch('largeur_axes_p(',inconnues_largeur)';
        if ~isempty(ind)
          %                 tmp = inconnues_largeur{ind(1)};
          %                 first = eval(tmp(16:end-1));
          %                 inds = first+ind-ind(1);
          tmp = regexp(cell2mat(inconnues_largeur(ind)),'\((\d+)\)','tokens');
          inds = str2num(char(cellfun(@char,tmp','UniformOutput',false)))';

          if length(ind) == Ninconnues % -> 1 seule inconnue pour tous les ind\ufffdfinis
            % inconnues_largeur = arraysub(inconnues_largeur,[],ind,ind);
            inconnues_largeur = {'largeur_axes_p_X'};
            largeur_axes_p_X = NaN;
            Ninconnues = 1;
            str = regexprep(int2str(inds),'\W*','|'); % on transforme les indices en string "1|2|4"
            equation_largeur = regexprep(equation_largeur,['largeur_axes_p\((',str,')\)'],'largeur_axes_p_X');
          else % il y a d'autres inconnues -> on bouche les trous sans se poser de question
            largeur_axes_p(inds) = Default_largeur_axes_p/M;
            % rem : cette section est incoh\ufffdrente \ufffd moins d'ajouter le param\ufffdtre AR_axes_p
            %             inds2 = find(any(~isnan(AR_axes(:,inds)'))&isnan(hauteur_axes)&isnan(hauteur_axes_p));
            %             if ~isempty(inds2)
            %               hauteur_axes_p(inds2) = largeur_axes_p(inds)*(1./AR_axes(inds2,inds)');
            %               str = regexprep(int2str(inds2),'\W*','|');
            %               inds2 = find(arrayfun(@isempty,regexp(inconnues_hauteur,['hauteur_axes_p(',str,')'])));
            %               inconnues_hauteur = arraysub(inconnues_hauteur,[],inds2,inds2); % enlever les inds2 de la liste des inconnues hauteur
            %             end
            Ninconnues = Ninconnues -length(inds);
            inconnues_largeur(ind) = [];
          end
        end
      end
    end

    % mettre \ufffd jour les hauteur_axes_p si AR_axes d\ufffdfini -> done

    % en hauteur :
    function fill_input_hauteur
      if length(inconnues_hauteur)>1 % -> remplissage par les valeurs par d\ufffdfaut
        Ninconnues = length(inconnues_hauteur);
        ind = strmatch('margeh_p',inconnues_hauteur,'exact');
        if Ninconnues >1 && ~isempty(ind)
          margeh_p = Default_margeh_p;
          inconnues_hauteur(ind) = [];
          Ninconnues = Ninconnues-1;
        end
        ind = strmatch('margeb_p',inconnues_hauteur,'exact');
        if Ninconnues >1 && ~isempty(ind)
          margeb_p = Default_margeb_p;
          inconnues_hauteur(ind) = [];
          Ninconnues = Ninconnues-1;
        end
        ind = strmatch('margey_p(',inconnues_hauteur)';
        if ~isempty(ind)
          %                 tmp = inconnues_hauteur{ind(1)};
          %                 first = eval(tmp(10:end-1));
          %                 inds = first+ind-ind(1);
          tmp = regexp(cell2mat(inconnues_hauteur(ind)),'\((\d+)\)','tokens');
          inds = str2num(char(cellfun(@char,tmp','UniformOutput',false)))';

          if length(ind) == Ninconnues % -> 1 seule inconnue pour tous les ind\ufffdfinis
            % inconnues_hauteur = arraysub(inconnues_hauteur,[],ind,ind);
            inconnues_hauteur = {'margey_p_X'};
            margey_p_X = NaN;
            Ninconnues = 1;
            str = regexprep(int2str(inds),'\W*','|'); % on transforme les indices en string "1|2|4"
            equation_hauteur = regexprep(equation_hauteur,['margey_p\((',str,')\)'],'margey_p_X');
          else % il y a d'autres inconnues -> on bouche les trous sans se poser de question
            margey_p(inds) = Default_margey_p/(N-1);
            Ninconnues = Ninconnues -length(inds);
            inconnues_hauteur(ind) = [];
          end
        end

        ind = strmatch('hauteur_axes_p(',inconnues_hauteur)';
        if ~isempty(ind)
          %                 tmp = inconnues_hauteur{ind(1)};
          %                 first = eval(tmp(16:end-1));
          %                 inds = first+ind-ind(1);
          tmp = regexp(cell2mat(inconnues_hauteur(ind)),'\((\d+)\)','tokens');
          inds = str2num(char(cellfun(@char,tmp','UniformOutput',false)))';
          if length(ind) == Ninconnues % -> 1 seule inconnue pour tous les ind\ufffdfinis
            % inconnues_hauteur = arraysub(inconnues_hauteur,[],ind,ind);
            inconnues_hauteur = {'hauteur_axes_p_X'};
            hauteur_axes_p_X = NaN;
            Ninconnues = 1;
            str = regexprep(int2str(inds),'\W*','|'); % on transforme les indices en string "1|2|4"
            equation_hauteur = regexprep(equation_hauteur,['hauteur_axes_p\((',str,')\)'],'hauteur_axes_p_X');
          else % il y a d'autres inconnues -> on bouche les trous sans se poser de question
            hauteur_axes_p(inds) = Default_hauteur_axes_p/N;
            % rem : cette section est incoh\ufffdrente \ufffd moins d'ajouter le param\ufffdtre AR_axes_p
            %             inds2 = find(any(~isnan(AR_axes(:,inds)'))&isnan(largeur_axes)&isnan(largeur_axes_p));
            %             if ~isempty(inds2)
            %               largeur_axes_p(inds2) = hauteur_axes_p(inds)*(AR_axes(inds2,inds)');
            %               str = regexprep(int2str(inds2),'\W*','|');
            %               inds2 = find(arrayfun(@isempty,regexp(inconnues_largeur,['largeur_axes_p(',str,')'])));
            %               inconnues_largeur = arraysub(inconnues_largeur,[],inds2,inds2); % enlever les inds2 de la liste des inconnues largeur
            %             end
            Ninconnues = Ninconnues -length(inds);
            inconnues_hauteur(ind) = [];
          end
        end
      end
    end

    % calcul des derni\ufffdres inconnues
    function calcul_equation_largeur()
      if length(inconnues_largeur)==1 % -> calcul de l'inconnue gr\ufffdce \ufffd l'\ufffdquation
        inconnue = char(inconnues_largeur);
        %             k = strfind(equation_largeur,inconnue);
        str = equation_largeur(1:end-1); % on vire le + \ufffd la fin
        str = regexprep(str,['(\W?)',regexptranslate('escape',inconnue),'(\W|$)'],'$1X$2');
        %str = strsub(str,'X',k,k+length(inconnnue)-1);
        eval([inconnue,' = fzero(@(X)',str,',0);']);
        switch inconnue
          case 'margex_p_X'
            ind = isnan(margex)&isnan(margex_p);
            margex_p(ind) = eval(inconnue);
          case 'largeur_axes_p_X'
            ind = isnan(largeur_axes)&isnan(largeur_axes_p);
            largeur_axes_p(ind) = eval(inconnue);
        end
      else % aucune inconnue -> test de consistance
      end
    end

    function calcul_equation_hauteur
      if length(inconnues_hauteur)==1 % -> calcul de l'inconnue gr\ufffdce \ufffd l'\ufffdquation
        inconnue = char(inconnues_hauteur);
        %             k = strfind(equation_hauteur,inconnue);
        str = equation_hauteur(1:end-1);
        str = regexprep(str,['(\W?)',regexptranslate('escape',inconnue),'(\W|$)'],'$1X$2');
        eval([inconnue,' = fzero(@(X)',str,',0);']);
        if inconnue < 0
          warning('splot:NegativeLength','WARNING! ONE OF THE COMPUTED DIMENSIONS IS NEGATIVE!')
        end
        switch inconnue
          case 'margey_p_X'
            ind = isnan(margey)&isnan(margey_p);
            margey_p(ind) = eval(inconnue);
          case 'hauteur_axes_p_X'
            ind = isnan(hauteur_axes)&isnan(hauteur_axes_p);
            hauteur_axes_p(ind) = eval(inconnue);
        end
      else % aucune inconnue -> test de consistance
        str = equation_hauteur(1:end-1);
        if eval(str) > 0
          warning('splot:Inconsistency','WARNING! THE HEIGHT OF THE FIGURE IS TOO SMALL!')
        elseif eval(str) < 0
          warning('splot:Inconsistency','WARNING! THE HEIGHT OF THE FIGURE IS TOO BIG!')
        end
      end
    end

    % passage de toutes les longueurs d\ufffdfinies par proportion en longueurs absolues
    
    function prop_2_abs_largeur
      clear liste;
      liste(1:2,:) = {'margeg','largeur','margeg_p';'marged','largeur','marged_p'};
      for j = 1:M-1
        liste(end+1,:) = {['margex(',int2str(j),')'],'largeur',['margex_p(',int2str(j),')']};
        liste(end+1,:) = {sprintf('largeur_axes(%u)',j),'largeur',sprintf('largeur_axes_p(%u)',j)};
      end
      liste(end+1,:) = {['largeur_axes(',int2str(M),')'],'largeur',['largeur_axes_p(',int2str(M),')']};

      %traitement
      for k = 1:size(liste,1)
        l1 = char(liste(k,1));
        l2 = char(liste(k,2));
        p = char(liste(k,3));
        s = find(isnan([eval(l1),eval(l2),eval(p)]));
        if length(s) > 1
          error('gros souci')
        elseif length(s) == 0
          %                 warning('splot:redondance','redondance dans les arguments')
        else
          if s == 1
            eval([l1,' = ',l2,'*',p,';']);
          end
        end
      end
    end

    function prop_2_abs_hauteur
      clear liste;
      liste(1:2,:) = {'margeh','hauteur','margeh_p';'margeb','hauteur','margeb_p'};
      for i = 1:N-1
        liste(end+1,:) = {['margey(',int2str(i),')'],'hauteur',['margey_p(',int2str(i),')']};
        liste(end+1,:) = {sprintf('hauteur_axes(%u)',i),'hauteur',sprintf('hauteur_axes_p(%u)',i)};
      end
      liste(end+1,:) = {['hauteur_axes(',int2str(N),')'],'hauteur',['hauteur_axes_p(',int2str(N),')']};


      %traitement
      for k = 1:size(liste,1)
        l1 = char(liste(k,1));
        l2 = char(liste(k,2));
        p = char(liste(k,3));
        s = find(isnan([eval(l1),eval(l2),eval(p)]));
        if length(s) > 1
          error('gros souci')
        elseif length(s) == 0
          %                 warning('splot:redondance','redondance dans les arguments')
        else
          if s == 1
            eval([l1,' = ',l2,'*',p,';']);
          end
        end
      end
    end
    
    
  end % fin de parseinput

end

